classdef Admin % in Admin.m
    methods (Static = true)
        
        % This class provides functions that speed up the
        % likelihood/filtering process 
        
        function coef_frac_mat = Convert_coef_frac_into_mat(coef_frac,k);
            % transform a vector of P values into k by k by P matrix
            nr_lags = length(coef_frac);
            coef_frac_mat = ones(k,k,nr_lags);
            for i  = 1:nr_lags
                coef_frac_mat(:,:,i) = coef_frac_mat(:,:,i) * coef_frac(i);
            end
        end

        function [phi_flip,phi_vec] =Frac_polynomal(d,beta,phi,nr_lags)
            
            % compute the infinite sum of weights of the FIGAS model
            % Note: this works only for the FIGAS(1,d,1) model
            % In the fashion of CONRAD & HAAG 2006
            % Conrad C., and B. R. Haag. 2006. Inequality Constraints in the Fractionally Integrated GARCH
            % Model. Journal of Financial Econometrics 4: 413–449.


            phi_vec    = zeros(nr_lags,1);
            g_vec      = zeros(nr_lags,1);
            c_vec      = zeros(nr_lags,1);
            f_vec      = zeros(nr_lags,1);

            f_1      = -d;
            g_vec(1) = f_1 * 1;
            c_vec(1) = beta + g_vec(1);
            phi_vec(1) = -c_vec(1) + phi*1;

            for i = 1:nr_lags-1
                f_vec(i+1)   = ((i+1)-1-d)/(i+1);
                g_vec(i+1)   = f_vec(i+1)*g_vec(i);

                c_vec(i+1)   = beta * c_vec(i) + g_vec(i+1);
                phi_vec(i+1) = -c_vec(i+1) + phi*c_vec(i);


            end

            phi_flip = flipud(phi_vec);

        end   
      
    end % end methods
end % end class