classdef DGP 
    methods (Static = true)
        
        %%
        function [s,f,V,P] = Store(k, T, V0);
                      
            s = zeros(k,k,T);
            f = zeros(k,k,T); f(:,:,1) = V0;
            V  = zeros(k, k, T); V(:,:,1) = V0;
            P  = zeros(k, k, T); P(:,:,1) = corrcov(V0);
            
        end
        
        
        %%
        
         function [RC, V] = get_FIGAS_MV_F_LM( k,T, params,V0)
            % This function simulates from the FIGAS model assuming a matrix F distribution.

            % INPUT
            % k:     number of assets
            % T:     number of obs
            % params vector of parameters of the model
            % V0     the unconditional mean of the matrix process
             
            % OUTPUT
            % RC     a k by k by T matrix of simulated realized covariances
            % V      a k by k by T matrix with filtered values of the
            %        varcov
            
            phi = 0;
            beta  = params(end-3);            
            nu_f1 = params(end-2);
            nu_f2 = params(end-1);
            d     = params(end);                        
            nr_lags = T-1;
            
            % compute the infinite sum given d, beta and phi
            coef_frac     = Admin.Frac_polynomal(d,beta,phi,nr_lags);
            coef_frac_mat = Admin.Convert_coef_frac_into_mat(coef_frac,k);                                                                                             
            
            % storage dims -> realized measures          
            RC = zeros(k, k, T);
            
            % score objects: score and GAS
            [s_star, ~, V] = Filter.Store(k, T, V0);                                            
            constant  = nu_f1/ (nu_f2-k-1);
            
            omega_mat = V0;
                                    
            for j=1:T
                V_j = V(:,:,j);                
                V_j = (V_j + V_j')/2;

                % simulate from matrix F
                temp_j= iwishrnd(V_j,nu_f2);
                RC(:,:,j) = (nu_f2-k-1) * wishrnd(temp_j/nu_f1,nu_f1);
                                
                % compute score
                invV = inv(V_j);
                d2 = nu_f1 * ((nu_f1 +nu_f2)/(nu_f2-k-1)*RC(:,:,j)*inv(eye(k)+ constant*invV*RC(:,:,j)) - V_j);                   
                
                nabla = d2/(nu_f1 + 1);                 
                s_star(:,:,j) = nabla + V_j - omega_mat;                              
                
                % update V
                if j<T                   
                   V_j_plus_1 = omega_mat + sum(coef_frac_mat(:,:,end-j+1:end).*s_star(:,:,1:j),3);                   
                   V(:,:,j+1) = (V_j_plus_1 + V_j_plus_1')/2;
                end
                
                
            end
                       
        end
 
        
    end % end methods
end % end class