classdef Filter % in Filter.m
    methods (Static = true)
        
        
        %%
        function [s, f, V] = Store(k, T, V0);
            s = zeros(k,k,T);
            f = zeros(k,k,T); f(:,:,1) = V0;
            V  = zeros(k, k, T); V(:,:,1) = V0;
            
            
        end
        
        function [V,s_star_DM,Flag,V_t,omega_mat,W_mat] = FIGAS_F_LM_tar( k, T, params,RC)            
            % This function computes the filtered var/cov matrices of the
            % FIGAS model
            
            % INPUT
            % k       (number of asset), T (number of obs)
            % params: vector of parameters
            % RC      a k by k by T matrix of pd matrices
           
            % OUTPUT
            % V          a k by k by T matrix of filtered varcov matrices
            % s_star DM  a k by k by T matrix of innovations to feed V
            % Flag       a 1/0 indicator function. 1 if V is nonPD
            % omega_mat  the unconditional varcov matrix (cov targeting)
            % W_mat      a k by k by T matrix of the score of the matrix F
            %            w.r.t V
            
            phi = 0;
           
            beta  = params(end-3);            
            nu_f1 = params(end-2);
            nu_f2 = params(end-1);
            d     = params(end);                        
            nr_lags = T-1;
            
            coef_frac     = Admin.Frac_polynomal(d,beta,phi,nr_lags);
            coef_frac_mat = Admin.Convert_coef_frac_into_mat(coef_frac,k);
                        
            V0 = mean(RC, 3);            
            omega_mat = V0;
            [s_star_DM, ~, V] = Filter.Store(k, T, V0);                                            
            constant  = nu_f1/ (nu_f2-k-1);
            Flag      = 0;
            
      
            W_mat  = zeros(k,k,T);
            
            for j=1:T                
                V_j      = V(:,:,j);
                
                [~,ind_pd] = chol(V_j);
                check_ind  = rcond(V(:,:,j));
                if check_ind < 1e-15 || ind_pd >0
                    Flag = 1;
                    break
                end
                                
                RC_j     = RC(:,:,j);
                invV     = inv( V_j );

                % scaled score from F dist      
                W_t      =  (nu_f1 +nu_f2)/(nu_f2-k-1)*inv(eye(k)+ constant*invV*RC_j);
                d2       = nu_f1 * (RC_j*W_t - V_j);        
                nabla = d2/(nu_f1 + 1);
                s_star_DM(:,:,j) = nabla + V_j - omega_mat;
                    
                W_mat(:,:,j) = W_t;
                
                if j<T                                       
                     V(:,:,j+1) = omega_mat + sum(coef_frac_mat(:,:,end-j+1:end).*s_star_DM(:,:,1:j),3);                 
                end
                
            end
            V_t = V(:,:,end);
            
        end                      
        
        
    end % end methods
end % end class
