classdef Loglike_MV % in Filter.m
    methods (Static = true)
        
        %% multi GAMMA in logs
        function [mg] = lnmultigamma(k, a)
            % return multivariate gamma
            mg= log(pi^(k*(k-1)/4));
            
            for i=1:k
                mg= mg + gammaln(a + (1-i)/2);
            end
            
        end
        
        
        %% multi GAMMA
        function [mg] = multigamma(k, a)
            % return multivariate gamma
            mg= pi^(k*(k-1)/4);
            
            for i=1:k
                mg= mg*gamma(a + (1-i)/2);
            end
            
        end
        
                    
                                                         
        function [LLF,loglik_RC,V] = LogLik_FIGAS_F_LM(k,T,params,RC);
                       
            % This function computes the loglikelihood of the FIGAS model
            % with covariance targeting, assuming a matrix F distribution 
            
            % INPUT
            % k, T          number of asset/ observations
            % params        vector with parameters of the FIGAS model
            % RC            a k by k by T matrix of realized covariance
            %               matrices
            
            % OUTPUT
            % LLF           the minus logl
            % loglik_RV     a T by 1 vector of matrix F loglikelihood values
            % V             a k by k by T matrix of a simulated path of
            %               time varying mean of the RC matrix according tot he FIGAS
            %               model
                        
            beta  = params(end-3);            
            nu_f1 = params(end-2);
            nu_f2 = params(end-1);
            d     = params(end);
            %c_2 = params(end);
            
            constant = nu_f1/ (nu_f2-k-1);                        
            [V,~,Flag]  = Filter.FIGAS_F_LM_tar(k, T, [beta nu_f1 nu_f2 d],RC);
                                    
            loglik_RC= zeros(T,1);
                       
            if Flag==1
                LLF = 1e12;                
                                             
            else
                
                for j=1:T
                    
                    j_RC         = RC(:,:,j);
                    j_invV       = inv(V(:,:,j));                    
                    j_logdetRC   = log(det(j_RC));
                    
                    % matrix F log likelihood (part that depend on t)
                    loglik_RC(j,1) = 0.5*nu_f1 * log(det(constant*j_invV)) + 0.5*(nu_f1-k-1)*j_logdetRC - 0.5*(nu_f1+nu_f2)*log(det(eye(k)+ constant*j_invV*j_RC));
                  
                end
                                                
                loglik_RC = loglik_RC + Loglike_MV.lnmultigamma(k, 0.5*(nu_f1+nu_f2)) - Loglike_MV.lnmultigamma(k, 0.5*nu_f1) - Loglike_MV.lnmultigamma(k, 0.5*nu_f2);                                                              
                LLF= -sum(loglik_RC);
                
                if  isnan(LLF) || ~isreal(LLF)
                    LLF=1e6;
                end
            end
            
        end
        
        
                             
        
        
        
        
        
        
    end
    
    
    
end


